

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

# Size of a pixel, in arcsec :
pixelscale = 0.008
#pixelscale = 0.016

# Dimensions in pixel of the field you want to calculate.
width = 1280 # pixel
height = 960 # pixel
#height = 1024 # pixel

#width = 640
#height = 480




# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 






import sys, os
import numpy as np
#import matplotlib.pyplot as plt
from matplotlib.mlab import griddata
import Image, ImageOps
# We write the gravlens input in a file :



# And we read the output :
print "Reading gravlens output..."
gd = np.loadtxt("grid.out") # grid data

print "Done."

# Columns: x1 y1 u1 v1 x2 y2 u2 v2 lev
#          0  1  2  3  4  5  6  7  8
# (x,y) in the image plane
# (u,v) in the source plane
#

print "Number of lines in grid : %i" % len(gd)

# We want to sort out identical lines as fast as possible.
# For this we start by sorting the grid data :
pts = gd[:,0:4].transpose()
pts = pts[:,np.lexsort(pts)].transpose()
# Now pts is like gd, but sorted according to last col, then second last col, etc...
# Usual trick : we calculate the diffs between the lines
diffs = pts[1:,] - pts[:-1,]
# Take the abs sum along the lines :
sumdiffs = np.sum(np.abs(diffs), axis=1)
# And keep all indices where sumdiffs is not zero :
keepindices = np.concatenate([np.array([True]), (sumdiffs > 0.000001)])
pts = pts[keepindices]
# Ok, now we have our independent points !
print "Number of independent points : %i" % len(pts)

x = pts[:,0]
y = pts[:,1]
u = pts[:,2]
v = pts[:,3]

minx = np.min(x)
maxx = np.max(x)
miny = np.min(y)
maxy = np.max(y)

print "Image plane extrema  : [% 8.3f, % 8.3f] x [% 8.3f, % 8.3f] [arcsec]" % (minx, maxx, miny, maxy)
print "Source plane extrema : [% 8.3f, % 8.3f] x [% 8.3f, % 8.3f] [arcsec]" % (np.min(u), np.max(u), np.min(v), np.max(v))


# So far so good. Now we want to interpolate these deflections on our pixel grid.


#pixelscale = 0.2
#width = 40
#height = 30

# The pixel grid on which to interpolate (in pixels)
ximin = (-width/2.0 +1.0)
ximax = (width/2.0)
yimin = (-height/2.0 +1.0)
yimax = (height/2.0)

xi = np.linspace(ximin, ximax, width)
yi = np.linspace(yimin, yimax, height)



"""
plt.contour(xi,yi,ui,15,linewidths=0.5,colors='k')
plt.contourf(xi,yi,ui,15,cmap=plt.cm.jet)
plt.colorbar() # draw colorbar
plt.scatter(x,y,marker='o',c='b',s=5)
plt.show()
"""


print "Requested sky field  : [% 8.3f, % 8.3f] x [% 8.3f, % 8.3f] [arcsec]" % (ximin*pixelscale, ximax*pixelscale, yimin*pixelscale, yimax*pixelscale)
#print "Requested sky field  : [% 8.3f, % 8.3f] x [% 8.3f, % 8.3f] [pixel]" % (ximin, ximax, yimin, yimax)

# we convert the image and source positions to pixels
x = x / pixelscale
y = y / pixelscale
u = u / pixelscale 
v = v / pixelscale

# and interpolate :
ui = griddata(x,y,u,xi,yi)
vi = griddata(x,y,v,xi,yi)

# and shift the origin to the lower left corner :
ui = ui - ximin
vi = vi - yimin
#vi = yimin - vi

print "Field source extrema : [% 8.1f, % 8.1f] x [% 8.1f, % 8.1f] [pixel]" % (np.min(ui), np.max(ui), np.min(vi), np.max(vi))

# We round to the nearest int (all this should be positive now) :
ui = np.cast['int'](np.around(ui))
vi = np.cast['int'](np.around(vi))

# We crop the interpolated source field to the field size.
ui = np.clip(ui, 0, width)
vi = np.clip(vi, 0, height)

print "Clipped to           : [% 8i, % 8i] x [% 8i, % 8i] [pixel]" % (np.min(ui), np.max(ui), np.min(vi), np.max(vi))


# Now what remains to be done is to encode this source field as rgb values of a uint8 png
# r = modulo 100
# g = multiple of 100
# b = 0

tens = 100*np.ones(ui.shape, dtype=np.int)
uir = np.cast['uint8'](np.fmod(ui, tens))
uig = np.cast['uint8'](((ui - uir) / 100))
uib = np.zeros(ui.shape, dtype=np.uint8)

vir = np.cast['uint8'](np.fmod(vi, tens))
vig = np.cast['uint8'](((vi - vir) / 100))
vib = np.zeros(vi.shape, dtype=np.uint8)

utest = uib + uig*100 + uir
vtest = vib + vig*100 + vir
if not (np.alltrue(utest - ui < 0.01) and np.alltrue(vtest - vi < 0.01)) :
	print "Encoding error !"
	sys.exit()



xdef = np.dstack((uir, uig, uib))
xdef = Image.fromarray(xdef, "RGB")
#xdef = ImageOps.flip(xdef)
xdef.save("xdef.png", "PNG")


ydef = np.dstack((vir, vig, vib))
ydef = Image.fromarray(ydef, "RGB")
#ydef = ImageOps.flip(ydef)
ydef.save("ydef.png", "PNG")






