from numpy.random import uniform, seed
from matplotlib.mlab import griddata
import matplotlib.pyplot as plt
import numpy as np
import Image, ImageOps

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

# Size of a pixel, in arcsec :
#pixelscale = 0.008
pixelscale = 0.016

# Dimensions in pixel of the field you want to calculate.
#width = 1280 # pixel
#height = 960 # pixel
#height = 1024 # pixel

width = 640
height = 480




# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 


infile = open("sources_delays.txt", "r")
lines = infile.readlines()
infile.close()

#mylines = [line.strip() for line in lines if (len(line) > 3 and "#" not in line and float(line.split()[3]) < 0.001)]
mylines = [line.strip() for line in lines if (len(line) > 3 and "#" not in line)]

print len(mylines)

x = np.array([float(line.split()[0]) for line in mylines])
y = np.array([float(line.split()[1]) for line in mylines])
z = np.array([float(line.split()[3]) for line in mylines])




# The pixel grid on which to interpolate (in pixels)
ximin = (-width/2.0 +1.0)
ximax = (width/2.0)
yimin = (-height/2.0 +1.0)
yimax = (height/2.0)

xi = np.linspace(ximin, ximax, width)
yi = np.linspace(yimin, yimax, height)




print "Requested sky field  : [% 8.3f, % 8.3f] x [% 8.3f, % 8.3f] [arcsec]" % (ximin*pixelscale, ximax*pixelscale, yimin*pixelscale, yimax*pixelscale)
#print "Requested sky field  : [% 8.3f, % 8.3f] x [% 8.3f, % 8.3f] [pixel]" % (ximin, ximax, yimin, yimax)

# we convert the image positions to pixels
x = x / pixelscale
y = y / pixelscale

# and interpolate :
zi = griddata(x,y,z,xi,yi)


# and shift the origin to the lower left corner :
xi = xi - ximin
yi = yi - yimin


print "Field extrema : [% 8.1f, % 8.1f] x [% 8.1f, % 8.1f] [pixel]" % (np.min(xi), np.max(xi), np.min(yi), np.max(yi))

# We scale the time delays ...

zi = 50.0 + zi * 20.0
#zi = 0.0 + zi * 30.0

# We round to the nearest int (all this should be positive now) :
zi = np.cast['int'](np.around(zi))

# Now what remains to be done is to encode this source field as rgb values into a png

zi = np.cast['uint8'](zi)


zirgb = np.dstack((zi, zi, zi))
ziimg = Image.fromarray(zirgb, "RGB")

ziimg = ImageOps.flip(ziimg)
ziimg.save("imagedelaymap.png", "PNG")

