"""
plot critical lines, caustics, source and images on a transparent png
"""


# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

# Size of a pixel, in arcsec :
pixelscale = 0.008

# Dimensions in pixel of the field you want to calculate.
width = 1280 # pixel
height = 960 # pixel
#height = 1024 # pixel

#width = 640
#height = 480


# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 






import sys, os
import numpy as np
#import matplotlib.pyplot as plt
#from matplotlib.mlab import griddata
import Image, ImageOps, ImageDraw
# We write the gravlens input in a file :



# And we read the output :
print "Reading gravlens output..."
gd = np.loadtxt("crit.out") # grid data

print "Done."

# Columns: x1 y1 u1 v1 x2 y2 u2 v2 lev
#          0  1  2  3  4  5  6  7  8
# (x,y) in the image plane
# (u,v) in the source plane
#

print "Number of lines in curves : %i" % len(gd)

critlines = gd[:,[0,1,4,5]]
critlines = critlines/pixelscale

caustlines = gd[:,[2,3,6,7]]
caustlines = caustlines/pixelscale

ximin = (-width/2.0 +1.0)
yimax = (height/2.0)


# We change the coordinate system to get 0 0 in the upper left ...
critlines[:, [0,2]] = critlines[:, [0,2]] - ximin # shift x1, x2
critlines[:, [1,3]] = yimax - critlines[:, [1,3]] # shift y1, y2

caustlines[:, [0,2]] = caustlines[:, [0,2]] - ximin # shift x1, x2
caustlines[:, [1,3]] = yimax - caustlines[:, [1,3]] # shift y1, y2


critlines = np.cast['int'](np.around(critlines))
caustlines = np.cast['int'](np.around(caustlines))


im = Image.new("RGBA", (width, height))




draw = ImageDraw.Draw(im)

for critline, caustline in zip(critlines, caustlines):
	draw.line([(critline[0], critline[1]), (critline[2], critline[3])], fill=(255, 255, 255), width=1)
	pass

# Now we draw some points...

# RXJ 1131
#source = np.array([1.504952e+00, 4.276876e-01])
#images = np.array([
#[3.115347e+00, 8.797984e-01],
#[-3.465345e-02, 1.187051e+00],
#[-1.464955e-03, -2.919858e-04],
#[5.929626e-01, -1.113738e+00],
#])

#source = np.array([-1.504952e+00, -4.276876e-01])
#images = np.array([
#[-3.115347e+00, -8.797984e-01],
#[+3.465345e-02, -1.187051e+00],
#[+1.464955e-03, +2.919858e-04],
#[-5.929626e-01, +1.113738e+00],
#])

source = np.array([4.950480e-01, 1.723124e-01])
images = np.array([
[-1.115346e+00, -2.798098e-01],
[ 2.034653e+00, -5.870840e-01],
[2.001404e+00, 6.005353e-01],
[1.407089e+00,  1.713674e+00],
])



source = source / pixelscale
images = images / pixelscale


source[0] = source[0] - ximin
source[1] = yimax - source[1]
images[:,0] = images[:,0] - ximin
images[:,1] = yimax - images[:,1]


rad = 6.0
#draw.ellipse([(source[0]-rad, source[1]-rad), (source[0]+rad, source[1]+rad)], outline=(0, 0, 255))
for img in images :
	draw.ellipse([(img[0]-rad, img[1]-rad), (img[0]+rad, img[1]+rad)], outline=(255, 0, 0))



del draw 


im.save("crit.png", "PNG")





